#include "ising.hpp"


void ising::remise_a_zero(bool mag)
{
  for (int i=0; i<N*N; i++)
	config[i] = mag; // Chaque site est réinitialisé avec la valeur mag.
}

void ising::tirage_aleatoire(std::mt19937& G)
{
  std::uniform_int_distribution<int> X(0,1);
  for (int i=0; i<N*N; i++)
	config[i] = X(G);
  // Chaque site est réinitialisé avec une valeur aléatoire (X(G) vaut
  // 1 ou 0 avec probabilité 1/2 - 1/2, ce qui est convertit en booléen
  // valant true ou false avec probabilité 1/2 - 1/2)
}

void theta::tirage_theta(int N, std::mt19937 &G)
{
  // Choix aléatoire d'un site 
  std::uniform_int_distribution<int> X(0,N-1);
  i = X(G);
  j = X(G);

  // u prend une valeur uniforme dans [0,1]
  std::uniform_real_distribution<double> U(0,1);
  u = U(G);
}


void ising::mise_a_jour(double beta, theta T)
{
  int delta = 0;

  // On calcule le δ de l'énoncé. Les "%N" servent à ramener dans
  // l'intervalle {0,...,N-1} les valeurs i-1, i+1, j-1, j+1 qui en
  // sortiraient (autrement dit, on travaille dans Z/NZ).
  delta += (*this)(T.I(),   (T.J()+1)%N) ? 1 : -1 ;
  delta += (*this)(T.I(),   (T.J()+N-1)%N) ? 1 : -1 ;
  delta += (*this)((T.I()+1)%N, T.J()  ) ? 1 : -1 ;
  delta += (*this)((T.I()+N-1)%N, T.J()  ) ? 1 : -1 ;

  // p correspond à la probabilité de mettre un + au site (i,j).
  double p = exp(beta*delta);
  p /= exp(beta*delta) + exp(-beta*delta);;
  
  if (T.U() < p)
	config[T.I()*N+T.J()] = true;
  else
	config[T.I()*N+T.J()] = false;
  }



std::ostream& operator<<(std::ostream &sortie, ising const &X)
{
  // La boucle en i correspond à l'affichage de chaque ligne,
  // la boucle en j à l'affichage de chaque caractère de la ième ligne.
  for (int i=0; i<X.taille(); i++)
	{
	  for (int j=0; j<X.taille(); j++)
		sortie << (X(i,j) ? "+" : "-");
	  sortie << "\n";
	}
  return sortie;
}

bool operator==(ising const &  X, ising const &  Y)
{
  // Si ls tailles des tableau X et Y sont différentes,
  // c'est que les tableaux sont différents... 
  if (X.N != Y.N)
	return false;

  // ... sinon, si X(i,j) et Y(i,j) sont distincts pour au un couple, c'est que X et Y sont différents.
  for (int i=0; i<X.N; i++)
	for (int j=0; j<X.N; j++)
	  if (X(i,j) != Y(i,j))
		return false;

  return true;
}

bool operator!=(ising const &  X, ising const &  Y)
{
  // Pas la peine de redétailler le test...
  return not(X==Y);
}


ising propp_wilson(int N, double beta, std::mt19937& gen)
{
  int n=1;
  
  ising X(N), Y(N);
  std::vector<theta> T;

  // On incrémente l'algorithme tant que la dynamique de Glauber ne
  // donne pas le même résultat partant d'un tableau de + ou d'un tableau de -.
  do
	{
	  T.push_back(theta()); // On ajoute un theta...
	  T[T.size()-1].tirage_theta(N,gen); // ... tiré aléatoirement

	  // De temps en temps (ici, à chaque temps de la forme 3^k), on calcule
	  // les configurations obtenues par les theta que l'ont a tiré, en partant
	  // des dex configurations initiales composées uniquement de + et de -.
	  if (T.size()==n)
		{
		  X.remise_a_zero(true);
		  Y.remise_a_zero(false);
		  
		  for (int k=T.size(); k!=0; k--)
			{
			  X.mise_a_jour(beta, T[k-1]);
			  Y.mise_a_jour(beta, T[k-1]);
			}
		  n *= 3;
		}
	  
	}
  
  while (X!=Y);
  
  return X;
}


