#include <iostream>
#include <random>
#include <vector>
#include <cmath>


// La classe theta, permettant de coder le choix d'un site et
// l'aléa décidant le signe qui sera affecté à ce site dans
// la dynamique de Glauber.
class theta
{
private:
  int i;
  int j;
  double u;
public:

  // Les accesseurs des trois membres.
  int I(void) const
  {	return i; }
  int J(void) const
  {	return j; }
  double U(void) const
  {	return u; }

  // La méthode permettant de tirer au hasard les trois valeurs i,j,u
  // (en prenant donc un générateur aléatoire)
  void tirage_theta(int N, std::mt19937 &G);
};



// La classe ising permettant de manipuler des configurations de + et -.
class ising
{
private:
  int N;
  std::vector<bool> config;
  
public:
  // Le constructeur par défaut crée un tableau de taille NxN, rempli avec la valeur "init" (qui vaut par défaut "true").
  ising(int N0, bool init=true) : N(N0), config(N*N,init) {}

  // Un accesseur sur la taille
  int taille(void) const
  { return N; }

  // Une méthode qui réinitialise le tableau entier avec une même valeur.
  void remise_a_zero(bool mag=true);

  // Une méthode qui choisit aléatoirement et indépendament les veleurs
  // chaque site
  void tirage_aleatoire(std::mt19937& G);

  // La mise à jour dans la dynamique de Glauber.
  void mise_a_jour(double beta, theta T);

  // Pour lire la valeur d'un des sites.
  bool operator()(int i, int j) const
  {
	return config[N*i+j];
  }

  // Les comparaisons.
  friend bool operator==(ising const  & X, ising const & Y);
  friend bool operator!=(ising const  & X, ising const & Y);


};
  

// L'affichage
std::ostream& operator<<(std::ostream &sortie, ising const & X);


// L'implémentation de l'algorithme de Propp-Wilson
ising propp_wilson(int N, double beta, std::mt19937& gen);

